package xn.update;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.Settings;
import android.text.Editable;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.CompoundButton;

import androidx.activity.EdgeToEdge;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;

import com.blankj.utilcode.util.ActivityUtils;
import com.blankj.utilcode.util.AppUtils;
import com.blankj.utilcode.util.GsonUtils;
import com.blankj.utilcode.util.LogUtils;
import com.blankj.utilcode.util.SPUtils;
import com.blankj.utilcode.util.ThreadUtils;
import com.blankj.utilcode.util.TimeUtils;
import com.blankj.utilcode.util.ToastUtils;
import com.hikvision.dmb.EthernetConfig;
import com.hikvision.dmb.LauncherInfo;
import com.hikvision.dmb.MemoryInfo;
import com.hikvision.dmb.TimeSwitchConfig;
import com.hikvision.dmb.display.InfoDisplayApi;
import com.hikvision.dmb.network.InfoNetworkApi;
import com.hikvision.dmb.system.InfoSystemApi;
import com.hikvision.dmb.time.InfoTimeApi;
import com.hikvision.dmb.util.InfoUtilApi;
import com.lxj.xpopupext.popup.TimePickerPopup;

import xn.update.constant.AppConstant;
import xn.update.databinding.ActivitySettingBinding;

@SuppressLint("SetTextI18n")
public class SettingActivity extends AppCompatActivity {
    private ActivitySettingBinding binding;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivitySettingBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        AsyncTask.execute(() -> runOnUiThread(() -> {
            // 状态栏
            binding.statusSwitch.setChecked(InfoDisplayApi.getStatusBarEnable());
            binding.statusSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> InfoDisplayApi.setStatusBarEnable(isChecked));
            // 导航栏
            binding.navSwitch.setChecked(InfoDisplayApi.getNavigationBarEnable());
            binding.navSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> InfoDisplayApi.setNavigationBarEnable(isChecked));
            // app版本
            binding.varBT.setText("版本:" + AppUtils.getAppVersionName());
            String sn = InfoSystemApi.getSerialNumber();
            // 设备sn
            binding.snBT.setText("SN:" + (TextUtils.isEmpty(sn) ? "NULL" : sn));
            // 网络ip
            EthernetConfig ethernetConfig = InfoNetworkApi.getEthernetConfig();
            String ip = ethernetConfig.getIpAddress();
            binding.ipBT.setText("Ip:" + (TextUtils.isEmpty(ip) ? "NULL" : ip));
            // 启动文件管理
            binding.fileBrowserBT.setOnClickListener(v -> Tool.INSTANCE.openFileBrowser());
            // 重启设备
            binding.rebootBT.setOnClickListener(v -> InfoSystemApi.reboot());
            // 自动拉起主应用
            binding.autoMasterSwitch.setChecked(SPUtils.getInstance().getBoolean(AppConstant.AUTO_MASTER, true));
            binding.autoMasterSwitch.setOnCheckedChangeListener((buttonView, isChecked)
                    -> SPUtils.getInstance().put(AppConstant.AUTO_MASTER, isChecked));
            // 自动更新主应用
            binding.autoUpdateSwitch.setChecked(SPUtils.getInstance().getBoolean(AppConstant.AUTO_UPDATE, true));
            binding.autoUpdateSwitch.setOnCheckedChangeListener((buttonView, isChecked)
                    -> SPUtils.getInstance().put(AppConstant.AUTO_UPDATE, isChecked));
            // 系统设置
            binding.settingBT.setOnClickListener(v -> {
                Tool.INSTANCE.openSetting();
            });
            // 超级权限
            if (SPUtils.getInstance().getBoolean("isRoot", false)) {
                binding.rootCV.setVisibility(View.VISIBLE);
            }
            // 温度
            binding.temperatureBT.setText("温度:" + InfoSystemApi.getTemperature());
            // CPU
            binding.cpuBT.setText(InfoSystemApi.getCpuUsageRate());
            // 内存
            MemoryInfo memoryInfo = InfoSystemApi.getMemoryUsage();
            binding.memoryBT.setText("内存:" + memoryInfo.describe);
            // http地址
            binding.httpUriET.setText(Tool.INSTANCE.getBaseUrl().toString());
            // 测试地址输入
            binding.httpUriTestBT.setOnClickListener(v -> binding.httpUriET.setText("http://192.168.1.8/api/"));
            // 西农地址输入
            binding.httpUriXnBT.setOnClickListener(v -> binding.httpUriET.setText("http://172.16.0.65/api/"));
            // 管理员密码
            binding.oldPasswordET.setText(SPUtils.getInstance().getString(AppConstant.ADMIN_PASSWORD, "admin@098&"));
            // 保存
            binding.save.setOnClickListener(v -> save());
            // 保存并启动主应用
            binding.saveAndStartMasterApp.setOnClickListener(v -> {
                if (save()) {
                    startMasterApp();
                }
            });
        }));
    }

    private boolean save() {
        try {
            Editable httpUriETText = binding.httpUriET.getText();
            if (null == httpUriETText || TextUtils.isEmpty(httpUriETText)) {
                binding.httpUriET.setError("请输入Http服务地址");
                return false;
            }
            Editable oldPasswordETText = binding.oldPasswordET.getText();
            if (null == oldPasswordETText || TextUtils.isEmpty(oldPasswordETText)) {
                binding.oldPasswordET.setError("请输入管理员密码");
                return false;
            }
            SPUtils.getInstance().put(AppConstant.BASE_URL, httpUriETText.toString());
            SPUtils.getInstance().put(AppConstant.ADMIN_PASSWORD, oldPasswordETText.toString());
            finish();
            return true;
        } catch (Exception e) {
            ToastUtils.showLong("参数异常无法保存,请核对参数!");
            LogUtils.e(Log.getStackTraceString(e));
        }
        return false;
    }

    private void startMasterApp() {
        Tool.INSTANCE.openApp("com.dlc.xn.eboard");
    }
}